==> ToolChain_STM32_1.0.50.tar.bz: ToolChain_STM32_1.0.50/ToolChain/scripts/compile.sh <==
#!/bin/bash

MAIN_C="$1"
NOLOG=""

AutomaticHeapName="000_automatic_heap_calculation"
AutomaticHeapMakefile="extensions.active/makefile.$AutomaticHeapName"
function createHeapMakefile() {
  HeapSize="$1"
  
cat <<END_OF_DYNAMIC_MAKEFILE >$AutomaticHeapMakefile
#{ EXTENSION_$AutomaticHeapName
#
# This makefile was automatically created by $0 during compile-time.
#

COMPILE_OPTS += -DEXTENSION_$AutomaticHeapName
COMPILE_OPTS += -DTTC_AUTOMATIC_HEAP_SIZE=$HeapSize

#}EXTENSION_$AutomaticHeapName
END_OF_DYNAMIC_MAKEFILE

}
function compileSources() {
  if [ "$1" == "LOGFILE" ]; then
    LOGFILE="1"
  else
    LOGFILE=""
  fi
  
  cat <<END_OF_HEADER >extensions.active/makefile #{
#
#                               ToolChain_STM32
#
# Central makefile.
# This makefile was created by concatenating all makefile.* files in this folder.
# 
# This file has been automatically created by $0 on `date`.
# ALL CHANGES MADE TO THIS FILE WILL GET LOST!
#

END_OF_HEADER
#}
  find extensions.active/makefile\.* -exec cat {} >>extensions.active/makefile \;
  
  ProjectFiles=`ls QtCreator/*.files`
  for ProjectFile in $ProjectFiles; do
    echo "`pwd`> ./_/updateProjectFiles.pl $ProjectFile ../ .."
    ./_/updateProjectFiles.pl $ProjectFile ../ >${ProjectFile}.log || Error="1"
  done
  if [ "$Error" == "1" ]; then
    echo "Compilation canceled due to error"
    exit 10
  fi
  
  Log="compile.log"
  CompileOK=""
  echo "compiling..."
  ls -l main.c
  make -j8 clean >/dev/null 2>&1
  if [ "$LOGFILE" == "1" ]; then
    make all -j1 >$Log 2>&1 && CompileOK="1"
  else
    if [ "$NOLOG" == "1" ]; then
      make all -j1 && CompileOK="1"
    else
      make all -j1 >$Log 2>&1 && CompileOK="1"
    fi
  fi
}

if [ "$USER" == "root" ]; then #{
  echo "$0 - ERROR: You should never start a compile process as user root. This would cause temporary files to be owned by root that are not accessible by normal user!"
  exit 10
fi #}
if [ "$MAIN_C" == "NOLOG" ]; then #{
  NOLOG="1"
  MAIN_C="$2"
else
  if [ "$2" == "NOLOG" ]; then
    NOLOG="1"
  fi
fi #}
if [ "$MAIN_C" != "" ]; then        #{ create link main.c
  if [ ! -e "mains/$MAIN_C" ]; then
    echo "$0 - ERROR: Cannot find main-file '$MAIN_C'!"
    echo "Available candidates: `ls -C mains/*.c`"
    exit 6
  fi
  rm 2>/dev/null main.c
  ln -s mains/$MAIN_C main.c
fi #}
if [ ! -e main.c ]; then            #{ check if main.c exists
  echo "$0 - ERROR: No main.c selected!"
  echo "Available candidates: `ls -C mains/*.c`"
  exit 5
fi #}
if [ ! -e compile_options.h ]; then #{ ensure this dynamic file exists
  touch compile_options.h
fi
if [ ! -e configs/memory.ld ]; then #{ ensure this dynamic file exists
  touch configs/memory.ld
fi #}

if [ -e $AutomaticHeapMakefile ]; then #{ first stage: compile with heap size = 1
  # set heap size to one byte for first run
  createHeapMakefile 1
fi #}
  
compileSources

if [ -e $AutomaticHeapMakefile ]; then #{ second stage: adjust heap-size + compile again
  if [ "$CompileOK" == "1" ]; then #{ calculate amount of non-static memory for use as heap
    
    # calculate total static memory usage
    StaticMemoryUsage=`./_/calculateMemoryUsage.pl  TOTAL`
    
    # read last word from ram-line of memory configuration
    RamSizeK=`grep " ram " configs/memory.ld | awk '{ print $(NF) }'`
    
    # remove trailing K + calculate bytes
    RamSize=`perl -e "print substr('$RamSizeK', 0, -1) * 1024;"`
    
    UnusedMemory=$(( $RamSize - $StaticMemoryUsage ))
  
    echo "$AutomaticHeapName: RamSize=$RamSize, StaticMemoryUsage=$StaticMemoryUsage, UnusedMemory=$UnusedMemory" #D
    
    # set heap size as big as possible
    createHeapMakefile $(( $UnusedMemory - 200 ))
    
    # recompile with new heap size
    compileSources LOGFILE
  
    if [ "$CompileOK" != "1" ]; then #{
      AmountOverflow=`grep "overflowed" compile.log | grep region | grep ram | awk '{ print $(NF-1) }'`
      
      if [ "$AmountOverflow" != "" ]; then
        echo "region ram overflowed by $AmountOverflow bytes: reducing heap size.."
        # set heap size as big as possible
        createHeapMakefile $(( $UnusedMemory - 200 - $AmountOverflow - 10 ))
        compileSources
      fi
    fi #}
    
  fi #}
fi #}

if [ "$CompileOK" == "1" ]; then #{
    if [ "$NOLOG" == "" ]; then #{ check log file for warnings + display with less
  
      grep warning $Log >compile.warnings
      Warnings=`cat compile.warnings`
      if [ "$Warnings" != "" ]; then
        cat <<END_OF_WARNINGS >>compile.warnings

Warnings occured. You should try to avoid these warnings. Sometimes a warning hints to a real problem.
Good code does compile without warnings.

Type Q to quit from less.
END_OF_WARNINGS
        LESS="-RX +/warning:"
        less compile.warnings
        
      fi
    fi #}
    
    for Binary in `ls *.elf`; do
      echo "creating assembly source of $Binary"
      arm-none-eabi-objdump -S --disassemble "$Binary" > "$Binary.S"
    done
    
    echo ""
    echo "Compiled successfully *************************************************"
    ls -lh *.bin *.elf *.S
    echo ""
fi #}

if [ "$CompileOK" != "1" ]; then #{ ERROR occured during compilation
    echo "ERROR occured during compilation!"
    if [ "$NOLOG" == "" ]; then #{
      cat <<END_OF_HINT >>$Log

Hint: You may run an analyze script on this log file to find out more about it!
      ./_/analyzeLogFile.pl $Log
      
END_OF_HINT
      LESS="-RX +/makefile:|make:|ld:|collect2:|error:|undefined reference|multiple definition|Assembler messages:";
      ${PAGER:-less} $Log
    fi #}
    exit 10
fi #}

echo ""

==> ToolChain_STM32_1.0.51.tar.bz: ToolChain_STM32_1.0.51/ToolChain/scripts/compile.sh <==
#!/bin/bash

MAIN_C="$1"
NOLOG=""

AutomaticHeapName="000_automatic_heap_calculation"
AutomaticHeapMakefile="extensions.active/makefile.$AutomaticHeapName"
function createHeapMakefile() {
  HeapSize="$1"
  
cat <<END_OF_DYNAMIC_MAKEFILE >$AutomaticHeapMakefile
#{ EXTENSION_$AutomaticHeapName
#
# This makefile was automatically created by $0 during compile-time.
#

COMPILE_OPTS += -DEXTENSION_$AutomaticHeapName
COMPILE_OPTS += -DTTC_AUTOMATIC_HEAP_SIZE=$HeapSize

#}EXTENSION_$AutomaticHeapName
END_OF_DYNAMIC_MAKEFILE

}
function compileSources() {
  if [ "$1" == "LOGFILE" ]; then
    LOGFILE="1"
  else
    LOGFILE=""
  fi
  
  cat <<END_OF_HEADER >extensions.active/makefile #{
#
#                               ToolChain_STM32
#
# Central makefile.
# This makefile was created by concatenating all makefile.* files in this folder.
# 
# This file has been automatically created by $0 on `date`.
# ALL CHANGES MADE TO THIS FILE WILL GET LOST!
#

END_OF_HEADER
#}
  find extensions.active/makefile\.* -exec cat {} >>extensions.active/makefile \;
  
  ProjectFiles=`ls QtCreator/*.files`
  for ProjectFile in $ProjectFiles; do
    echo "`pwd`> ./_/updateProjectFiles.pl $ProjectFile ../ .."
    ./_/updateProjectFiles.pl $ProjectFile ../ >${ProjectFile}.log || Error="1"
  done
  if [ "$Error" == "1" ]; then
    echo "Compilation canceled due to error"
    exit 10
  fi
  
  Log="compile.log"
  CompileOK=""
  echo "compiling..."
  ls -l main.c
  make -j8 clean >/dev/null 2>&1
  if [ "$LOGFILE" == "1" ]; then
    make all -j1 >$Log 2>&1 && CompileOK="1"
  else
    if [ "$NOLOG" == "1" ]; then
      make all -j1 && CompileOK="1"
    else
      make all -j1 >$Log 2>&1 && CompileOK="1"
    fi
  fi
}

if [ "$USER" == "root" ]; then #{
  echo "$0 - ERROR: You should never start a compile process as user root. This would cause temporary files to be owned by root that are not accessible by normal user!"
  exit 10
fi #}
if [ "$MAIN_C" == "NOLOG" ]; then #{
  NOLOG="1"
  MAIN_C="$2"
else
  if [ "$2" == "NOLOG" ]; then
    NOLOG="1"
  fi
fi #}
if [ "$MAIN_C" != "" ]; then        #{ create link main.c
  if [ ! -e "mains/$MAIN_C" ]; then
    echo "$0 - ERROR: Cannot find main-file '$MAIN_C'!"
    echo "Available candidates: `ls -C mains/*.c`"
    exit 6
  fi
  rm 2>/dev/null main.c
  ln -s mains/$MAIN_C main.c
fi #}
if [ ! -e main.c ]; then            #{ check if main.c exists
  echo "$0 - ERROR: No main.c selected!"
  echo "Available candidates: `ls -C mains/*.c`"
  exit 5
fi #}
if [ ! -e compile_options.h ]; then #{ ensure this dynamic file exists
  touch compile_options.h
fi
if [ ! -e configs/memory.ld ]; then #{ ensure this dynamic file exists
  touch configs/memory.ld
fi #}

if [ -e $AutomaticHeapMakefile ]; then #{ first stage: compile with heap size = 1
  # set heap size to one byte for first run
  createHeapMakefile 1
fi #}
  
compileSources

if [ -e $AutomaticHeapMakefile ]; then #{ second stage: adjust heap-size + compile again
  if [ "$CompileOK" == "1" ]; then #{ calculate amount of non-static memory for use as heap
    
    # calculate total static memory usage
    StaticMemoryUsage=`./_/calculateMemoryUsage.pl  TOTAL`
    
    # read last word from ram-line of memory configuration
    RamSizeK=`grep " ram " configs/memory.ld | awk '{ print $(NF) }'`
    
    # remove trailing K + calculate bytes
    RamSize=`perl -e "print substr('$RamSizeK', 0, -1) * 1024;"`
    
    UnusedMemory=$(( $RamSize - $StaticMemoryUsage ))
  
    echo "$AutomaticHeapName: RamSize=$RamSize, StaticMemoryUsage=$StaticMemoryUsage, UnusedMemory=$UnusedMemory" #D
    
    # set heap size as big as possible
    createHeapMakefile $(( $UnusedMemory - 200 ))
    
    # recompile with new heap size
    compileSources LOGFILE
  
    if [ "$CompileOK" != "1" ]; then #{
      AmountOverflow=`grep "overflowed" compile.log | grep region | grep ram | awk '{ print $(NF-1) }'`
      
      if [ "$AmountOverflow" != "" ]; then
        echo "region ram overflowed by $AmountOverflow bytes: reducing heap size.."
        # set heap size as big as possible
        createHeapMakefile $(( $UnusedMemory - 200 - $AmountOverflow - 10 ))
        compileSources
      fi
    fi #}
    
  fi #}
fi #}

if [ "$CompileOK" == "1" ]; then #{
    if [ "$NOLOG" == "" ]; then #{ check log file for warnings + display with less
  
      grep -v warning $Log | grep -- "-warning" >compile.warnings
      Warnings=`cat compile.warnings`
      if [ "$Warnings" != "" ]; then
        cat <<END_OF_WARNINGS >>compile.warnings

Warnings occured. You should try to avoid these warnings. Sometimes a warning hints to a real problem.
Good code does compile without warnings.

Type Q to quit from less.
END_OF_WARNINGS
        LESS="-RX +/warning:"
        less compile.warnings
        
      fi
    fi #}
    
    for Binary in `ls *.elf`; do
      echo "creating assembly source of $Binary"
      arm-none-eabi-objdump -S --disassemble "$Binary" > "$Binary.S"
    done
    
    echo ""
    echo "Compiled successfully *************************************************"
    ls -lh *.bin *.elf *.S
    echo ""
fi #}

if [ "$CompileOK" != "1" ]; then #{ ERROR occured during compilation
    echo "ERROR occured during compilation!"
    if [ "$NOLOG" == "" ]; then #{
      cat <<END_OF_HINT >>$Log

Hint: You may run an analyze script on this log file to find out more about it!
      ./_/analyzeLogFile.pl $Log
      
END_OF_HINT
      LESS="-RX +/makefile:|make:|ld:|collect2:|error:|undefined reference|multiple definition|Assembler messages:";
      ${PAGER:-less} $Log
    fi #}
    exit 10
fi #}

echo ""
